<?php

namespace Drupal\Tests\languages_dropdown\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the Languages Dropdown module.
 *
 * @group languages_dropdown
 */
class LanguagesDropdownAdminTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'locale', 'language', 'block', 'languages_dropdown'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Disable strict config schema.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create and log in user.
    $admin_user = $this->drupalCreateUser([
      'administer blocks',
      'administer languages',
      'administer site configuration',
      'access administration pages',
      'access content',
    ]);
    $this->drupalLogin($admin_user);
  }

  /**
   * Tests if user can load the front page among other things.
   */
  public function testLoadSite() {
    // Load the front page.
    $this->drupalGet('<front>');

    // Confirm that the site didn't throw a server error or something else.
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet("user");
  }

  /**
   * Tests if an admin user can log in.
   */
  public function testAdminUser() {
    // Log in an administrative user.
    $this->drupalLogin($this->rootUser);

    $this->drupalGet('admin/config');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
  }

  /**
   * Functional tests for the language switcher block.
   */
  public function testLanguageDropdownBlock(): void {
    // Add language.
    $edit = [
      'predefined_langcode' => 'es',
    ];
    $this->drupalGet('admin/config/regional/language/add');
    $this->submitForm($edit, 'Add language');

    // Set the native language name.
    $this->saveNativeLanguageName('en', 'english');

    // Enable URL language detection and selection.
    $edit = ['language_interface[enabled][language-url]' => '1'];
    $this->drupalGet('admin/config/regional/language/detection');
    $this->submitForm($edit, 'Save settings');

    // Clear all caches before looking if the new jQCloud block was created.
    drupal_flush_all_caches();
  }

  /**
   * Adds the Languages Dropdown Menu.
   */
  public function testBlockVisibility() {
    $assert_session = $this->assertSession();

    $this->drupalGet('/admin/structure/block');
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('Block');
    $assert_session->pageTextContains('Header');
    $assert_session->responseContains('id="edit-blocks-region-header-title"');

    $this->drupalGet('/admin/structure/block/library/' . $this->defaultTheme);
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('Block');
    $assert_session->pageTextContains('Category');
    $assert_session->pageTextContains('Operations');
    $assert_session->pageTextContains('Languages Dropdown');

    $this->drupalGet('/admin/structure/block/add/lang_drop_bootstrap%3Alanguage_interface/' . $this->defaultTheme);

    $assert_session->statusCodeEquals(200);
    $assert_session->buttonExists("Save block");

    $edit = [
      'region' => 'content',
    ];

    $this->submitForm($edit, 'Save block');
    $assert_session->pageTextContains('The block configuration has been saved.');

    drupal_flush_all_caches();

    $this->drupalGet('<front>');
    $assert_session->statusCodeEquals(200);
  }

  /**
   * Tests if an admin access to pages again.
   */
  public function testAdminUserAfterChanges() {
    // Log in an administrative user.
    $this->drupalLogin($this->rootUser);

    $this->drupalGet('admin/modules');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
  }

  /**
   * Saves the native name of a language entity in configuration as a label.
   *
   * @param string $langcode
   *   The language code of the language.
   * @param string $label
   *   The native name of the language.
   */
  protected function saveNativeLanguageName($langcode, $label): void {
    \Drupal::service('language.config_factory_override')
      ->getOverride($langcode, 'language.entity.' . $langcode)->set('label', $label)->save();
  }

}
